'' =================================================================================================
''
''   File....... __activity_board.spin
''   Purpose....  
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated....  
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                           ' use 5MHz crystal

  CLK_FREQ = _xinfreq * (_clkmode >> 6)                          ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                    ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                                ' ticks in 1us
                                                                  
                                                                  
con { pab io pins }                                               
                                                                  
  RX1     = 31                                                   ' programming / terminal
  TX1     = 30                                      
                                                    
  SDA     = 29                                                   ' eeprom / i2c
  SCL     = 28                                      
                                                      
  AUD_RT  = 27                                                   ' audio (d/a) outputs
  AUD_LF  = 26                                             
                                                      
  LED_27  = 27                                                   ' leds (on audio outputs)
  LED_26  = 26                                             
                                                      
  SD_CS   = 25                                                   ' usd card
  SD_DI   = 24                                                    
  SD_CLK  = 23
  SD_DO   = 22

  ADC_CS  = 21                                                   ' adc input
  ADC_CLK = 20                                                    
  ADC_DO  = 19                               
  ADC_DI  = 18                                
                                                                  
  SVO_6   = 17                                                   ' servo headers
  SVO_5   = 16
  SVO_4   = 15
  SVO_3   = 14
  SVO_2   = 13
  SVO_1   = 12
    

con 

  #(-1), IS_ON, IS_OFF


obj

' main                                                           ' * master spin cog
  time  : "jm_time"                                              '   delays/timing
  prng  : "jm_prng"                                              '   random #s
  io    : "jm_io"                                                '   basic io
  adc   : "jm_adc124s021"                                        '   for onboard adc
  term  : "jm_fullduplexserial"                                  ' * serial for terminal
  servo : "jm_servo8"                                            ' * servo handling
  sd    : "fsrw"                                                 ' * uSD file IO
                                                                  
' * uses cog when loaded
  

var


pub main 

  setup

  repeat
    waitcnt(0)


pub setup

  time.start                                                     ' setup for timing/delays

  prng.seed(cnt, cnt ~> 2, $DEAD_BEEF, cnt << 2, cnt)            ' setup randomizer                             

  io.start(0, 0)                                                 ' clear pins (this cog)

' adc.start(ADC_CS, ADC_CLK, ADC_DI, ADC_DO)                     ' connect to adc 
  
  term.start(RX1, TX1, %0000, 115_200)                           ' start serial for terminal *
                                                                  
' servo.start(6, SVO_1)                                          ' start servo driver *
                                                                  
  mount_sd                                                       ' start sd access *


con

  { ----------------- }
  {  F I L E   I / O  } 
  { ----------------- }


var

  long  mounted

  byte  filename[8 + 1 + 3 + 1]                                  ' 8.3 plus 0
  byte  linebuf[80 + 1]                                          ' line buffer
  

pub mount_sd | check

'' Attempt to mount SD card
'' -- returns true if SD card successfully mounted

  check := \sd.mount_explicit(SD_DO, SD_CLK, SD_DI, SD_CS)

  mounted := (check == 0)   

  return mounted


pub dismount_sd

'' Unmounts SD card

  \sd.unmount
  mounted := false


pub has_file(p_str) | check

'' Returns true if file is on SD card

  check := \sd.popen(p_str, "r")
  if (check == 0)
    return true
  else
    return false 


pub open_file(p_str, mode) | check

'' Attempt to open file
'' -- p_str is pointer to filename (z-string)
'' -- mode is [lower-case] single character, (e.g., "r" for read)
'' -- sd card must be mounted before open

  if (mounted)
    check := \sd.popen(p_str, mode)                              ' attempt to open
    if (check == 0)                                               
      if (mode == "r")                     
        \sd.seek(0)                                              ' force sector read 
      return true
    else
      return false
  else
    return false


pub close_file

'' Closes open file

  \sd.pclose


pub rd_line(p_str, n) | c, len

'' Reads line of text from open text file
'' -- terminated by LF or EOF
'' -- p_str is address of string buffer
'' -- n is maximum number of characters to read from line

  len := 0                                                       ' index into string
                                                                  
  repeat n                                                        
    c := \sd.pgetc                                               ' get a character
    if (c < 0)                                                   ' end of file
      if (len == 0)                                              ' if nothing
        return -1                                                ' return empty
      else                                                        
        quit                                                      
                                                                  
    if (c <> 10)                                                 ' if not LF char
      byte[p_str++] := c                                         '  add to buffer
      len++                                                       
    else                                                         ' if LF
      quit                                                       '  we're done
                                                                  
  byte[p_str] := 0                                               ' terminate end of line
                                                                  
  return len
  

pub wr_line(p_str, usecr, uself)

'' Writes z-string to open text file
'' -- set usecr to true to add CR
'' -- set uself to true to add LF

  \sd.pputs(p_str)
  if (usecr)
    \sd.pputc(13)
  if (uself)
    \sd.pputc(10)


pub rd_buf(p_buf, n) | check

'' Reads n bytes from file to buffer at p_buf
'' -- returns number of bytes read

  return \sd.pread(p_buf, n)


pub wr_buf(p_buf, n) | check   

'' Writes n bytes to file from buffer at p_buf
'' -- returns number of bytes written

  return \sd.pwrite(p_buf, n)


pub directory | fcount, check, idx, p_fname, fsize

'' Show directory (root only) of uSD card

  sd.opendir                                                     ' setup for .nextfile
  fcount := 0                                                    ' reset files count
                                                                  
  term.str(string("Directory", term#CR, term#CR))                 
                                                                  
  repeat                                                          
    bytefill(@filename, 0, 13)                                    
    check := \sd.nextfile(@filename)                             ' get next file in root
    if (check == 0)                                              ' if found
      fcount += 1                                                '  increment file count
      term.str(@filename)                                        '  show name
      term.tx(term#CR)                                            
    else                                                         ' else
      quit                                                       '  done
                                                                  
  if (fcount > 0)
    term.tx(term#CR)
    term.dec(fcount)
    term.str(string(" file(s) found."))
  else
    term.str(string("No files found."))

  sd.pclose


pub show_file(p_file) | len

'' Display contents of text file

  ifnot (mounted)
    term.str(string("SD not mounted.", 13))
    return

  if (open_file(p_file, "r"))
    repeat
      bytefill(@linebuf, 0, 81)
      len := rd_line(@linebuf, 80)
      if (len > 0)
        term.str(@linebuf)
        time.pause(5)
      else
        close_file
        quit 
    

dat { license }

{{

  Copyright (c) 2015 Jon McPhalen  

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}